///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Jul 31, 2006	Created
//    TF Jul 13, 2008   Adapted "View" for TRIPview 2.0 and renamed to View2.
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.data;

import java.io.*;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.data.*;
import com.tietoenator.trip.jxp.logging.*;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * This class demonstrates how to interact with TRIPview functionality, i.e.
 * extracting text from binary files, and rendering a binary object into one
 * or more formats for viewing. This example class demonstrates use of 
 * functionality compatible with TRIPview-C and TRIPview 2.x.
 */
public class View2
{

   private TdbSession m_session;
   private String m_database = null;
   private String m_stringField = null;
   private String m_textField = null;
   private String m_pnameField = null;
   private String m_pvalueField = null;

   /**
    * Constructor
    * @throws TdbException
    */
   public View2()
           throws TdbException
   {
      // Create a session on a physical server (no grid support)
      m_session = SessionFactory.newInstance("**** TRIPview example ****", false);
      m_session.setActivityLog(new TdbConsoleLogger(true));
   }

   private void setDatabase(String db)
   {
      m_database = db;
   }

   private void setStringField(String sf)
   {
      m_stringField = sf;
   }

   private void setTextField(String tf)
   {
      m_textField = tf;
   }

   private void setPNameField(String pn)
   {
      m_pnameField = pn;
   }

   private void setPValueField(String pn)
   {
      m_pvalueField = pn;
   }

   /**
    * Perform text extraction or HTML conversion as specified.
    */
   public void process()
           throws Exception
   {
      boolean brk = false;
      while (!brk)
      {
         TdbRecord rec = null;

         Console stdin = new Console();
         int mode = -1;

         // Prompt the user to store or retrieve a record.
         while (mode == -1)
         {
            mode = stdin.readint("Mode (0 = store, 1 = retrieve)", 0, true);
            if (mode == 1 && m_stringField == null)
            {
               System.out.println("\nRetrieval mode is not permitted if you haven't specified a string field!\n");
               mode = -1;
            }
         }

         switch (mode)
         {
         case 0:
         {
            String name = null;
            while (name == null)
            {
               name = stdin.readln("File name", null);
            }

            rec = new TdbRecord(m_session, m_database, false);

            TdbTextField fld = (TdbTextField) rec.getHead().createField(m_textField, TdbFieldType.TextField);
            TdbTextExtractionInfo txi = fld.getTextExtractionInfo();

            // The optional string field into which the file is to be stored.
            if (m_stringField != null)
            {
               txi.setBinaryCopyField(m_stringField);
            }

            // The optional phrase field that receieves property name/value pairs.
            if (m_pnameField != null && m_pvalueField != null)
            {
               txi.setPropertyNameField(m_pnameField);
               txi.setPropertyValueField(m_pvalueField);
            }

            // While not strictly necessary, the name of the file is valuable to the text
            // extractor when it tries to determine the file type.
            txi.setFileName(name);

            // Read file data using a stream. Note that if TRIP is located on
            // the same machine as the TRIP application, the FileName property
            // (as set above) is sufficient. Passing file data via a stream
            // to JXP should only be done if 1) the file is in memory only,
            // or 2) if TRIP and the application are running on two different
            // machines.
            txi.setStream(new FileInputStream(name));

            // Remember to explicitly state that text extraction is to be performed.
            txi.setExtractText(true);

            // Send the request to the server.
            rec.commit();
            break;
         }

         case 1:
         {
            int recnr = stdin.readint("Record #", 1, true);
            int rend = stdin.readint("Rendition (0-none, 1-HTML, 2-MHT)", 0, true);
            int hlight = stdin.readint("Highlighting (0-No, 1-Yes)", 0, true);

            String strSearchTerm = null;
            if (hlight != 0)
            {
               while (null == strSearchTerm)
               {
                  strSearchTerm = stdin.readln("Term to search for", null);
               }
            }

            TdbFieldTemplate fld = null;
            TdbRenditionType rtype = null;

            // Set appropriate rendition type according to user request
            if (rend == 1)
            {
               rtype = TdbRenditionType.BasicHTML;
            }
            else if (rend == 2)
            {
               rtype = TdbRenditionType.MimeEncodedHtml;
            }

            // Create the appropriate retrieval template
            if (rend > 0 && hlight != 0)
            {
               fld = new TdbHighlightRendition(m_stringField, m_textField, rtype);
            }
            else if (rend > 0 && hlight == 0)
            {
               fld = new TdbRendition(m_stringField, rtype);
            }
            else if (rend == 0)
            {
               fld = new TdbFieldTemplate(m_stringField);
            }
            else
            {
               brk = true;
            }


            if (!brk)
            {
               // Highlight not selected - fetch requested record using its RID.
               if (0 == hlight)
               {
                  rec = new TdbRecord(m_session, m_database, false);
                  rec.setRecordId(recnr);
                  rec.addToTemplate(fld);
                  rec.get();
               }
               // Highlight requested - perform search to get some hit locations
               else
               {
                  // Prepare record set for query
                  TdbRecordSet rs = new TdbRecordSet(m_session);
                  rs.setDatabase(m_database);
                  rs.setQuery("FIND R=" + Integer.toString(recnr) + " AND " + strSearchTerm);
                  rs.setFrom(1);
                  rs.setTo(1);

                  // Specify the retreival template
                  TdbRecord tmplr = new TdbRecord(m_session, m_database, false);
                  tmplr.addToTemplate(fld);
                  rs.setRetrievalTemplate(tmplr);

                  // Perform search and retrieval
                  rs.get();

                  if (rs.getQueryRecords() < 1)
                  {
                     System.out.println("No match! Please try again.");
                     brk = true;
                  }
                  else
                  {
                     rec = rs.recordAt(0);
                  }
               }

               // If we got a record, prompt for name of file to save the result to.
               if (rec != null)
               {
                  String name = null;
                  while (name == null)
                  {
                     name = stdin.readln("File name", null);
                  }
                  TdbStringField str = (TdbStringField) rec.getHead().getField(m_stringField);
                  str.copyToFile(name);
               }
            }
            break;
         }

         default:
            brk = true;
         }
      }

      // Terminate the session
      m_session.logout();
   }

   /**
    * Main entry point
    * @param args Not used
    */
   public static void main(String[] args)
   {
      try
      {
         View2 vw = new View2();
         String db = null, txt = null, pname = null, pval = null;
         Console stdin = new Console();

         // Get a valid database name, and set the list of
         // retrieval fields from it
         while (db == null)
         {
            db = stdin.readln("Database", null);
         }
         while (txt == null)
         {
            txt = stdin.readln("Text field", null);
         }

         vw.setDatabase(db);
         vw.setTextField(txt);

         vw.setStringField(stdin.readln("String field (optional)", null));

         pname = stdin.readln("Phrase field for property names (opt)", null);
         if (pname != null)
         {
            vw.setPNameField(pname);
            while (pval == null)
            {
               pval = stdin.readln("Phrase field for property values", null);
            }
            vw.setPValueField(pval);
         }

         // Perform the operation
         vw.process();
      }
      catch (TdbException e)
      {
         System.out.println(e.getDebugMessage());
      }
      catch (Exception e)
      {
         e.printStackTrace();
      }
   }
}
