///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2018 infinIT Services GmbH
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//        TF    Jul 19, 2018    Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.facet;

import com.tietoenator.trip.jxp.TdbException;
import com.tietoenator.trip.jxp.TdbMessageProvider;
import com.tietoenator.trip.jxp.examples.util.Console;
import com.tietoenator.trip.jxp.examples.util.SessionFactory;
import com.tietoenator.trip.jxp.data.TdbSearch;
import com.tietoenator.trip.jxp.data.TdbSearchSet;
import com.tietoenator.trip.jxp.facet.TdbFacet;
import com.tietoenator.trip.jxp.facet.TdbFacetSet;
import com.tietoenator.trip.jxp.facet.TdbFacetSortOrder;
import com.tietoenator.trip.jxp.facet.TdbFacetValue;
import com.tietoenator.trip.jxp.facet.TdbFieldTermFacet;
import com.tietoenator.trip.jxp.session.TdbSession;

/**
 * Example of how to use baselining with the facet classes in TRIPjxp.
 */
public class FacetBaseline
{
    private TdbSession m_session;
    
    public FacetBaseline()
    throws TdbException
    {
        // Create a session on a physical server (no grid support)
        m_session = SessionFactory.newInstance("**** TRIP Facet Baselinings Example ****", false);
    }
    
    public void close()
    {
        if (m_session != null)
        {
            
            try
            {
                m_session.logout();
            }
            catch (TdbException ignore)
            {
            }
            finally
            {
                m_session = null;
            }
        }
    }
    
    
    // Show any success message that might be forthcoming from the server
    
    private static void showMessages(TdbMessageProvider prv)
    {
        for (String msg : prv.messageList())
        {
            System.out.println(msg);
        }
        prv.resetMessages();
    }
    
    
    private void printFacetValues(TdbFacet f)
    {
        System.out.println();
        System.out.println(f.getName()+":");
        for (TdbFacetValue v : f)
        {
            System.out.print("["+Integer.toString(v.getRecordCount()));
            
            // The isBaselineValid property can be used to check if it makes sense
            // to retrieve and display the baseline count.
            if (v.isBaselineValid())
                System.out.print("/"+Integer.toString(v.getBaselineCount())+"]\t");
            else
                System.out.print("]\t");
            
            System.out.println(v.toString());
        }
    }
    
    private TdbFacetSet createFacets(TdbSearchSet sset, boolean regBaseline, boolean useBaseline)
    throws TdbException
    {
        // Create a couple of facets and assign those to a facet set.
        // Set baseline recall to 2000 terms. Should ideally be as low as
        // possible, though, even lower than this. Any additional terms
        // will not be recalled, which means that subsequent facets using
        // the baseline will only get an estimated baseline count for such
        // terms.
        
        TdbFacetSet facetset = new TdbFacetSet(m_session);
        
        TdbFieldTermFacet facet1 = new TdbFieldTermFacet("SPEAKER", "#");
		if (regBaseline)
		{
			facet1.setRegisterBaseline(regBaseline);
			facet1.setBaselineSize(2000);
		}
		else if (useBaseline)
		{
			facet1.setUseBaseline(useBaseline);
		}
        facet1.setSearchSet(sset.getSearchId());
        facet1.setFetchFrom(1);
        facet1.setFetchTo(10);
        facet1.setSortOrder(TdbFacetSortOrder.FrequencyDescending);
        facetset.add(facet1);
        
        TdbFieldTermFacet facet2 = new TdbFieldTermFacet("PERSON", "#");
		if (regBaseline)
		{
			facet2.setRegisterBaseline(regBaseline);
			facet2.setBaselineSize(2000);
		}
		else if (useBaseline)
		{
			facet2.setUseBaseline(useBaseline);
		}
        facet2.setSearchSet(sset.getSearchId());
        facet2.setFetchFrom(1);
        facet2.setFetchTo(10);
        facet2.setSortOrder(TdbFacetSortOrder.FrequencyDescending);
        facetset.add(facet2);
        
        return facetset;
    }
    
    private void process(Console stdin) throws TdbException
    {
        // A common scenario is to first open a database and perhaps
        // do a (full text) search.
        
        TdbSearch search = new TdbSearch(m_session);
        search.execute("BASE ALICE");
        search.execute("FIND ALICE");
        TdbSearchSet sset = search.getLastSearchSet();
        
        // Retrieve the facet values and set them as baseline
        TdbFacetSet facetset = createFacets(sset,true,false);
        facetset.get();
        
        // Show first round of results
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx));
        }
        
        // Assume the user has selected a facet in person, so
        // here we emulate a search refinement scenario.
        search.execute("FIND ALICE AND PERSON='WHITE KING'");
        sset = search.getLastSearchSet();
        
        // Issue a new identical facet request and tell it to use the previous
        // one as a baseline. Any facet values missing from the new set will
        // be inserted from the baseline.
        facetset.close();
        facetset = createFacets(sset,false,true);
        facetset.get();
        
        System.out.println();
        System.out.println("Facets with baseline information");
        System.out.println();
        
        // Show second round of results
        for (int facetIdx=0;
             facetIdx<facetset.getFacetCount();
             facetIdx++)
        {
            printFacetValues(facetset.getFacet(facetIdx));
        }
        
        // Remember to close the facet set immediately when you're done with it.
        // Since we have used baselines, we should also remove those when they are
        // no longer needed in the session. Note that baselines will be automatically
        // deleted when new ones are registered.
        facetset.removeBaselines();
        facetset.close();
    }
    
    
    public static void main(String[] args)
    {
        FacetBaseline app = null;
        Console stdin = new Console();
        try
        {
            app = new FacetBaseline();
            app.process(stdin);
        }
        catch (TdbException tx)
        {
            System.err.println(tx.getDebugMessage());
        }
        catch (Exception ox)
        {
            ox.printStackTrace(System.err);
        }
        finally
        {
            if (app != null)
                app.close();
        }
    }
}

