///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2008-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		TF	Mar 18, 2008	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.pool;

import com.tietoenator.trip.jxp.*;
import com.tietoenator.trip.jxp.ccl.*;
import com.tietoenator.trip.jxp.logging.*;
import com.tietoenator.trip.jxp.session.*;
import com.tietoenator.trip.jxp.pool.*;
import com.tietoenator.trip.jxp.examples.util.*;

public class SessionPool implements TdbPoolInitializer
{

   public SessionPool()
   {
   }

   /**
    * Demonstrates how to create a session pool.
    *
    * @param prompt A text prompt to be shown on screen before any other stuff
    * @param stepping How many sessions that are to be created at a time.
    * @return A session pool object.
    */
   private TdbConnectionPool createPool(String prompt, int stepping) throws TdbException
   {
      TdbConnectionPool pool = null;
      Console stdin = new Console();
      String type, host, username, password;
      char ct;
      int dfltport, port;

      System.out.println(prompt + "\n");
      for (;;)
      {
         type = stdin.readln("Session type (net, web)", "net");
         if (type == null)
         {
            return null;
         }
         ct = type.charAt(0);
         if (ct != 'n' && ct != 'w')
         {
            continue;
         }

         if (ct == 'n')
         {
            dfltport = 23457;
         }
         else
         {
            dfltport = 80;
         }

         host = stdin.readln("Server", "localhost");
         port = stdin.readint("Port#", dfltport, true);
         username = stdin.readln("Username", null);
         password = stdin.readln("Password", null);

         // Connect to the server using the appropriate protocol
         switch (ct)
         {
         case 'n':
            pool = new TdbTripNetConnectionPool(host, port, username, password, TdbLanguage.English, stepping);
            break;
         case 'w':
            pool = new TdbWebConnectionPool(host, port, username, password, TdbLanguage.English, stepping);
            break;
         }
         break;
      }
      
      if (pool != null)
      {
         pool.setPoolInitializer(this);
      }
      
      return pool;
   }

   public void initializePooledSession(TdbPooledSession session) throws TdbException
   {
      /* All calls that apply to the entire lifetime of a pooled session
       * should be put here. E.g. opening databases, defines, etc.
       */
  
	  TdbCclCommand cmd = new TdbCclCommand(session);
	  cmd.execDirect("DEFINE FIND MIN=0");
      System.out.println("   Session initialized: ID="+session.hashCode());
   }

   public static void main(String[] args)
   {
      TdbSession session = null;
      TdbConnectionPool pool = null;
      SessionPool example = null;
      java.util.List<String> products = null;
      Console stdin = new Console();

      try
      {
         // Create a session pool
         example = new SessionPool();
         pool = example.createPool("**** Simple session pool example ****", 2);
		 
		 if (stdin.readbool("Pool logging",false,true))
			pool.setActivityLog(new TdbConsoleLogger(true));

		 System.out.println("Setting up purge thread");
		 
         // Set up purging of idle sessions.
         // NOTE: These values are for demo purposes only. You really should
         // use higher values for production, setting the idleTimeout value
         // slightly lower than the purgeInterval. So you could for example
         // use: idleTimeout=55 and purgeInterval=60.
         pool.setIdleTimeout(5);
         pool.setPurgeInterval(10);
         pool.setMinIdle(5);
		 
         // Set max limit for the pool
         pool.setLimit(10);
		 
		 System.out.println("Reserving up to limit of "+Integer.toString(pool.getLimit())+" sessions");

         // Make sure the pool has enough sessions. This extends the pool
         // up to the limit. It only throws an exception if no sessions at
         // all could be reserved.
         pool.reserve(20);

         // If all is ok, the pool should now contain 10 sessions.
         System.out.println("The pool is of size " + Integer.toString(pool.getSize()));

         // Get a session from the pool with infinite timeout.
         session = (TdbSession) pool.acquire(TdbConnectionPool.INFINITE_TIMEOUT);

		 System.out.println("Session acquired");
		 
         // Do something in the session; here - just display
         // a list of the TRIP products the server we're connecting
         // to has a license for.
         products = session.licensedProducts();
         System.out.println("Licensed TRIP products:");
         for (String product : products)
         {
            System.out.println("   " + product);
         }

         // Restore the session to the pool
         session.logout();
         session = null;

         System.out.println("Waiting 15 seconds to give purge routine a chance to run.");
         Thread.sleep(15000);
         System.out.println("The pool is now of size " + Integer.toString(pool.getSize()));

         // Close the session pool
         pool.close();
      }
      catch (Exception x)
      {
         x.printStackTrace();
      }
   }
}
