///////////////////////////////////////////////////////////////////////////////
//
// TRIPjxp Programming Examples
//
// Copyright (C) 2006-2009 Tieto Sweden AB - All Rights Reserved
//
///////////////////////////////////////////////////////////////////////////////
//
// Modification History:
//		GF	Jul 17, 2006	Created
//
///////////////////////////////////////////////////////////////////////////////
package com.tietoenator.trip.jxp.examples.util;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import com.tietoenator.trip.jxp.TdbException;

/**
 * Simple console wrapper, provides a means to prompt for responses and validate
 * those responses (in some small ways)
 */
public class Console
{

   private BufferedReader m_stdin;
   private String m_spaces = "                                                  ";

   /**
    * Constructor
    */
   public Console()
   {
      m_stdin = new BufferedReader(new InputStreamReader(System.in));
   }

   /**
    * Prompt the user, get their response. If no response is given (i.e. the
    * user just hits RETURN), the default provided is returned.
    *
    * @param prefix Prompt to show to the user (this is padded to 40 characters)
    * @param dflt Default response if no other is forthcoming from the user
    * @return The response (or the default)
    * @throws TdbException
    */
   public String readln(String prefix, String dflt)
           throws TdbException
   {
      return readln(prefix, dflt, true);
   }

   /**
    * Prompt the user, get their response. If no response is given (i.e. the
    * user just hits RETURN), the default provided is returned.
    *
    * @param prefix Prompt to show to the user
    * @param dflt Default response if no other is forthcoming from the user
    * @param pad If true, the prompt is padded to 50 characters
    * @return The response (or the default)
    * @throws TdbException
    */
   public String readln(String prefix, String dflt, boolean pad)
           throws TdbException
   {
      if (pad)
      {
         int slen = prefix.length();
         int dlen = dflt != null && dflt.length() > 0 ? dflt.length() + 2 : 0;
         int avail = 48 - dlen;

         if (slen >= avail)
         {
            System.out.print(prefix.substring(0, avail));
         }
         else
         {
            System.out.print(prefix);
            System.out.print(m_spaces.substring(0, avail - slen));
         }
         if (dlen > 0)
         {
            System.out.print("[" + dflt + "]");
         }
         System.out.print(": ");
      }
      else
      {
         if (dflt != null && dflt.length() > 0)
         {
            System.out.print(prefix + " [" + dflt + "] ");
         }
         else
         {
            System.out.print(prefix + " ");
         }
      }

      try
      {
         String val = m_stdin.readLine();
         if (val == null)
         {
            return null;
         }
         val = val.trim();
         if (val.length() == 0) //  && dflt != null )
         {
            return dflt;
         }
         return val;
      }
      catch (IOException e)
      {
         throw new TdbException(TdbException.COMMUNICATIONS_ERROR, e);
      }
   }

   /**
    * Prompt the user, get their response, ensure that the response is an
    * integer. If no response is given (i.e. the user just hits RETURN), the
    * default provided is returned.
    *
    * @param prefix Prompt to show to the user
    * @param dflt Default response if no other is forthcoming from the user
    * @param pad If true, the prompt is padded to 40 characters
    * @return The response (or the default)
    * @throws TdbException
    */
   public int readint(String prefix, int dflt, boolean pad)
           throws TdbException
   {
      String dfltstr = String.valueOf(dflt);
      for (;;)
      {
         try
         {
            String val = readln(prefix, dfltstr, pad);
            return Integer.valueOf(val).intValue();
         }
         catch (Exception e)
         {
         }
      }
   }

   /**
    * Prompt the user, get their response, ensure that the response is a
    * boolean. If no response is given (i.e. the user just hits RETURN), the
    * default provided is returned.
    *
    * @param prefix Prompt to show to the user
    * @param dflt Default response if no other is forthcoming from the user
    * @param pad If true, the prompt is padded to 40 characters
    * @return The response (or the default)
    * @throws TdbException
    */
   public boolean readbool(String prefix, boolean dflt, boolean pad)
           throws TdbException
   {
      String dfltstr = dflt ? "Y" : "N";
      for (;;)
      {
         try
         {
            String val = readln(prefix, dfltstr, pad);
            return (val.equalsIgnoreCase("y") ||
                    val.equalsIgnoreCase("t") || 
                    val.equals("1") ||
                    val.equalsIgnoreCase("yes") ||
                    val.equalsIgnoreCase("true"));
         }
         catch (Exception e)
         {
         }
      }
   }
}
